@@ -7,6 +7,7 @@ import (
 	"fmt"
 	"os"
 	"path/filepath"
+	"slices"
 	"sync"
 	"time"
 
@@ -27,9 +28,10 @@ var (
 func snapshotFromModel(m model) historyEntry {
 	return historyEntry{
 		Path:          m.path,
-		Entries:       cloneDirEntries(m.entries),
-		LargeFiles:    cloneFileEntries(m.largeFiles),
+		Entries:       slices.Clone(m.entries),
+		LargeFiles:    slices.Clone(m.largeFiles),
 		TotalSize:     m.totalSize,
+		TotalFiles:    m.totalFiles,
 		Selected:      m.selected,
 		EntryOffset:   m.offset,
 		LargeSelected: m.largeSelected,
@@ -44,24 +46,6 @@ func cacheSnapshot(m model) historyEntry {
 	return entry
 }
 
-func cloneDirEntries(entries []dirEntry) []dirEntry {
-	if len(entries) == 0 {
-		return nil
-	}
-	copied := make([]dirEntry, len(entries))
-	copy(copied, entries) //nolint:all
-	return copied
-}
-
-func cloneFileEntries(files []fileEntry) []fileEntry {
-	if len(files) == 0 {
-		return nil
-	}
-	copied := make([]fileEntry, len(files))
-	copy(copied, files) //nolint:all
-	return copied
-}
-
 func ensureOverviewSnapshotCacheLocked() error {
 	if overviewSnapshotLoaded {
 		return nil
@@ -250,6 +234,7 @@ func saveCacheToDisk(path string, result scanResult) error {
 		Entries:    result.Entries,
 		LargeFiles: result.LargeFiles,
 		TotalSize:  result.TotalSize,
+		TotalFiles: result.TotalFiles,
 		ModTime:    info.ModTime(),
 		ScanTime:   time.Now(),
 	}
@@ -264,6 +249,29 @@ func saveCacheToDisk(path string, result scanResult) error {
 	return encoder.Encode(entry)
 }
 
+// peekCacheTotalFiles attempts to read the total file count from cache,
+// ignoring expiration. Used for initial scan progress estimates.
+func peekCacheTotalFiles(path string) (int64, error) {
+	cachePath, err := getCachePath(path)
+	if err != nil {
+		return 0, err
+	}
+
+	file, err := os.Open(cachePath)
+	if err != nil {
+		return 0, err
+	}
+	defer file.Close() //nolint:errcheck
+
+	var entry cacheEntry
+	decoder := gob.NewDecoder(file)
+	if err := decoder.Decode(&entry); err != nil {
+		return 0, err
+	}
+
+	return entry.TotalFiles, nil
+}
+
 func invalidateCache(path string) {
 	cachePath, err := getCachePath(path)
 	if err == nil {
