@@ -10,8 +10,6 @@ import (
 	"strconv"
 	"strings"
 	"time"
-
-	"github.com/shirou/gopsutil/v3/host"
 )
 
 var (
@@ -227,20 +225,42 @@ func collectThermal() ThermalStatus {
 				valStr, _, _ = strings.Cut(valStr, ",")
 				valStr, _, _ = strings.Cut(valStr, "}")
 				valStr = strings.TrimSpace(valStr)
-				if powerMW, err := strconv.ParseFloat(valStr, 64); err == nil && powerMW > 0 {
-					thermal.SystemPower = powerMW / 1000.0
+				if powerMW, err := strconv.ParseFloat(valStr, 64); err == nil {
+					// SystemPower should always be positive, reject invalid values
+					if powerMW >= 0 && powerMW < 1000000 { // 0 to 1000W
+						thermal.SystemPower = powerMW / 1000.0
+					}
 				}
 			}
 
-			// Battery power (mW -> W, positive = discharging).
+			// Battery power (mW -> W, positive = discharging, negative = charging).
 			if _, after, found := strings.Cut(line, "\"BatteryPower\"="); found {
 				valStr := strings.TrimSpace(after)
 				valStr, _, _ = strings.Cut(valStr, ",")
 				valStr, _, _ = strings.Cut(valStr, "}")
 				valStr = strings.TrimSpace(valStr)
-				// Parse as int64 first to handle negative values (charging)
-				if powerMW, err := strconv.ParseInt(valStr, 10, 64); err == nil {
-					thermal.BatteryPower = float64(powerMW) / 1000.0
+
+				var powerMW float64
+				var parsed bool
+
+				// Strategy 1: Try parsing as a signed integer first.
+				// This handles standard positive values and explicit negative strings like "-12345".
+				if valInt, err := strconv.ParseInt(valStr, 10, 64); err == nil {
+					powerMW = float64(valInt)
+					parsed = true
+				} else if valUint, err := strconv.ParseUint(valStr, 10, 64); err == nil {
+					// Strategy 2: Try parsing as an unsigned integer (Two's Complement).
+					// ioreg often returns negative values as huge uint64 numbers (e.g. 2^64 - 100).
+					// Casting such a uint64 to int64 correctly restores the negative value.
+					powerMW = float64(int64(valUint))
+					parsed = true
+				}
+
+				if parsed {
+					// Validate reasonable battery power range: -200W to 200W
+					if powerMW > -200000 && powerMW < 200000 {
+						thermal.BatteryPower = powerMW / 1000.0
+					}
 				}
 			}
 		}
@@ -261,29 +281,3 @@ func collectThermal() ThermalStatus {
 
 	return thermal
 }
-
-func collectSensors() ([]SensorReading, error) {
-	temps, err := host.SensorsTemperatures()
-	if err != nil {
-		return nil, err
-	}
-	var out []SensorReading
-	for _, t := range temps {
-		if t.Temperature <= 0 || t.Temperature > 150 {
-			continue
-		}
-		out = append(out, SensorReading{
-			Label: prettifyLabel(t.SensorKey),
-			Value: t.Temperature,
-			Unit:  "°C",
-		})
-	}
-	return out, nil
-}
-
-func prettifyLabel(key string) string {
-	key = strings.TrimSpace(key)
-	key = strings.TrimPrefix(key, "TC")
-	key = strings.ReplaceAll(key, "_", " ")
-	return key
-}
