@@ -201,33 +201,18 @@ func getScoreStyle(score int) lipgloss.Style {
 	}
 }
 
-func buildCards(m MetricsSnapshot, _ int) []cardData {
-	cards := []cardData{
-		renderCPUCard(m.CPU),
-		renderMemoryCard(m.Memory),
-		renderDiskCard(m.Disks, m.DiskIO),
-		renderBatteryCard(m.Batteries, m.Thermal),
-		renderProcessCard(m.TopProcesses),
-		renderNetworkCard(m.Network, m.Proxy),
-	}
-	if hasSensorData(m.Sensors) {
-		cards = append(cards, renderSensorsCard(m.Sensors))
-	}
-	return cards
-}
-
-func hasSensorData(sensors []SensorReading) bool {
-	for _, s := range sensors {
-		if s.Note == "" && s.Value > 0 {
-			return true
-		}
-	}
-	return false
-}
-
-func renderCPUCard(cpu CPUStatus) cardData {
+func renderCPUCard(cpu CPUStatus, thermal ThermalStatus) cardData {
 	var lines []string
-	lines = append(lines, fmt.Sprintf("Total  %s  %5.1f%%", progressBar(cpu.Usage), cpu.Usage))
+
+	// Line 1: Usage + Temp (Format: 15% @ 30.4°C)
+	usageBar := progressBar(cpu.Usage)
+
+	headerText := fmt.Sprintf("%5.1f%%", cpu.Usage)
+	if thermal.CPUTemp > 0 {
+		headerText += fmt.Sprintf(" @ %s°C", colorizeTemp(thermal.CPUTemp))
+	}
+
+	lines = append(lines, fmt.Sprintf("Total  %s  %s", usageBar, headerText))
 
 	if cpu.PerCoreEstimated {
 		lines = append(lines, subtleStyle.Render("Per-core data unavailable (using averaged load)"))
@@ -408,6 +393,22 @@ func renderProcessCard(procs []ProcessInfo) cardData {
 	return cardData{icon: iconProcs, title: "Processes", lines: lines}
 }
 
+func buildCards(m MetricsSnapshot, width int) []cardData {
+	cards := []cardData{
+		renderCPUCard(m.CPU, m.Thermal),
+		renderMemoryCard(m.Memory),
+		renderDiskCard(m.Disks, m.DiskIO),
+		renderBatteryCard(m.Batteries, m.Thermal),
+		renderProcessCard(m.TopProcesses),
+		renderNetworkCard(m.Network, m.NetworkHistory, m.Proxy, width),
+	}
+	// Sensors card disabled - redundant with CPU temp
+	// if hasSensorData(m.Sensors) {
+	// 	cards = append(cards, renderSensorsCard(m.Sensors))
+	// }
+	return cards
+}
+
 func miniBar(percent float64) string {
 	filled := min(int(percent/20), 5)
 	if filled < 0 {
@@ -416,7 +417,7 @@ func miniBar(percent float64) string {
 	return colorizePercent(percent, strings.Repeat("▮", filled)+strings.Repeat("▯", 5-filled))
 }
 
-func renderNetworkCard(netStats []NetworkStatus, proxy ProxyStatus) cardData {
+func renderNetworkCard(netStats []NetworkStatus, history NetworkHistory, proxy ProxyStatus, cardWidth int) cardData {
 	var lines []string
 	var totalRx, totalTx float64
 	var primaryIP string
@@ -432,10 +433,23 @@ func renderNetworkCard(netStats []NetworkStatus, proxy ProxyStatus) cardData {
 	if len(netStats) == 0 {
 		lines = []string{subtleStyle.Render("Collecting...")}
 	} else {
-		rxBar := netBar(totalRx)
-		txBar := netBar(totalTx)
-		lines = append(lines, fmt.Sprintf("Down   %s  %s", rxBar, formatRate(totalRx)))
-		lines = append(lines, fmt.Sprintf("Up     %s  %s", txBar, formatRate(totalTx)))
+		// Calculate dynamic width
+		// Layout: "Down   " (7) + graph + "  " (2) + rate (approx 10-12)
+		// Safe margin: 22 chars.
+		// We target 16 chars to match progressBar implementation for visual consistency.
+		graphWidth := cardWidth - 22
+		if graphWidth < 5 {
+			graphWidth = 5
+		}
+		if graphWidth > 16 {
+			graphWidth = 16 // Match progressBar fixed width
+		}
+
+		// sparkline graphs
+		rxSparkline := sparkline(history.RxHistory, totalRx, graphWidth)
+		txSparkline := sparkline(history.TxHistory, totalTx, graphWidth)
+		lines = append(lines, fmt.Sprintf("Down   %s  %s", rxSparkline, formatRate(totalRx)))
+		lines = append(lines, fmt.Sprintf("Up     %s  %s", txSparkline, formatRate(totalTx)))
 		// Show proxy and IP on one line.
 		var infoParts []string
 		if proxy.Enabled {
@@ -451,19 +465,54 @@ func renderNetworkCard(netStats []NetworkStatus, proxy ProxyStatus) cardData {
 	return cardData{icon: iconNetwork, title: "Network", lines: lines}
 }
 
-func netBar(rate float64) string {
-	filled := min(int(rate/2.0), 5)
-	if filled < 0 {
-		filled = 0
+// 8 levels: ▁▂▃▄▅▆▇█
+func sparkline(history []float64, current float64, width int) string {
+	blocks := []rune{'▁', '▂', '▃', '▄', '▅', '▆', '▇', '█'}
+
+	data := make([]float64, 0, width)
+	if len(history) > 0 {
+		// Take the most recent points.
+		start := 0
+		if len(history) > width {
+			start = len(history) - width
+		}
+		data = append(data, history[start:]...)
 	}
-	bar := strings.Repeat("▮", filled) + strings.Repeat("▯", 5-filled)
-	if rate > 8 {
-		return dangerStyle.Render(bar)
+	// padding with zeros at the start
+	for len(data) < width {
+		data = append([]float64{0}, data...)
 	}
-	if rate > 3 {
-		return warnStyle.Render(bar)
+	if len(data) > width {
+		data = data[len(data)-width:]
 	}
-	return okStyle.Render(bar)
+
+	maxVal := 0.1
+	for _, v := range data {
+		if v > maxVal {
+			maxVal = v
+		}
+	}
+
+	var builder strings.Builder
+	for _, v := range data {
+		level := int((v / maxVal) * float64(len(blocks)-1))
+		if level < 0 {
+			level = 0
+		}
+		if level >= len(blocks) {
+			level = len(blocks) - 1
+		}
+		builder.WriteRune(blocks[level])
+	}
+
+	result := builder.String()
+	if current > 8 {
+		return dangerStyle.Render(result)
+	}
+	if current > 3 {
+		return warnStyle.Render(result)
+	}
+	return okStyle.Render(result)
 }
 
 func renderBatteryCard(batts []BatteryStatus, thermal ThermalStatus) cardData {
@@ -527,12 +576,7 @@ func renderBatteryCard(batts []BatteryStatus, thermal ThermalStatus) cardData {
 		}
 
 		if thermal.CPUTemp > 0 {
-			tempText := fmt.Sprintf("%.0f°C", thermal.CPUTemp)
-			if thermal.CPUTemp > 80 {
-				tempText = dangerStyle.Render(tempText)
-			} else if thermal.CPUTemp > 60 {
-				tempText = warnStyle.Render(tempText)
-			}
+			tempText := colorizeTemp(thermal.CPUTemp) + "°C" // Reuse common color logic
 			healthParts = append(healthParts, tempText)
 		}
 
@@ -548,20 +592,6 @@ func renderBatteryCard(batts []BatteryStatus, thermal ThermalStatus) cardData {
 	return cardData{icon: iconBattery, title: "Power", lines: lines}
 }
 
-func renderSensorsCard(sensors []SensorReading) cardData {
-	var lines []string
-	for _, s := range sensors {
-		if s.Note != "" {
-			continue
-		}
-		lines = append(lines, fmt.Sprintf("%-12s %s", shorten(s.Label, 12), colorizeTemp(s.Value)+s.Unit))
-	}
-	if len(lines) == 0 {
-		lines = append(lines, subtleStyle.Render("No sensors"))
-	}
-	return cardData{icon: iconSensors, title: "Sensors", lines: lines}
-}
-
 func renderCard(data cardData, width int, height int) string {
 	titleText := data.icon + " " + data.title
 	lineLen := max(width-lipgloss.Width(titleText)-2, 4)
@@ -641,12 +671,12 @@ func colorizeBattery(percent float64, s string) string {
 
 func colorizeTemp(t float64) string {
 	switch {
-	case t >= 85:
+	case t >= 76:
 		return dangerStyle.Render(fmt.Sprintf("%.1f", t))
-	case t >= 70:
+	case t >= 56:
 		return warnStyle.Render(fmt.Sprintf("%.1f", t))
 	default:
-		return subtleStyle.Render(fmt.Sprintf("%.1f", t))
+		return okStyle.Render(fmt.Sprintf("%.1f", t))
 	}
 }
 
