@@ -5,78 +5,13 @@ clean_user_essentials() {
     start_section_spinner "Scanning caches..."
     safe_clean ~/Library/Caches/* "User app cache"
     stop_section_spinner
-    start_section_spinner "Scanning empty items..."
-    clean_empty_library_items
-    stop_section_spinner
+
     safe_clean ~/Library/Logs/* "User app logs"
-    if is_path_whitelisted "$HOME/.Trash"; then
-        note_activity
-        echo -e "  ${GREEN}${ICON_EMPTY}${NC} Trash · whitelist protected"
-    else
+    if ! is_path_whitelisted "$HOME/.Trash"; then
         safe_clean ~/.Trash/* "Trash"
     fi
 }
 
-clean_empty_library_items() {
-    if [[ ! -d "$HOME/Library" ]]; then
-        return 0
-    fi
-
-    # 1. Clean top-level empty directories in Library
-    local -a empty_dirs=()
-    while IFS= read -r -d '' dir; do
-        [[ -d "$dir" ]] && empty_dirs+=("$dir")
-    done < <(find "$HOME/Library" -mindepth 1 -maxdepth 1 -type d -empty -print0 2> /dev/null)
-
-    if [[ ${#empty_dirs[@]} -gt 0 ]]; then
-        safe_clean "${empty_dirs[@]}" "Empty Library folders"
-    fi
-
-    # 2. Clean empty subdirectories in Application Support and other key locations
-    # Iteratively remove empty directories until no more are found
-    local -a key_locations=(
-        "$HOME/Library/Application Support"
-        "$HOME/Library/Caches"
-    )
-
-    for location in "${key_locations[@]}"; do
-        [[ -d "$location" ]] || continue
-
-        # Limit passes to keep cleanup fast; 3 iterations handle most nested scenarios.
-        local max_iterations=3
-        local iteration=0
-
-        while [[ $iteration -lt $max_iterations ]]; do
-            local -a nested_empty_dirs=()
-            # Find empty directories
-            while IFS= read -r -d '' dir; do
-                # Skip if whitelisted
-                if is_path_whitelisted "$dir"; then
-                    continue
-                fi
-                # Skip protected system components
-                local dir_name=$(basename "$dir")
-                if is_critical_system_component "$dir_name"; then
-                    continue
-                fi
-                [[ -d "$dir" ]] && nested_empty_dirs+=("$dir")
-            done < <(find "$location" -mindepth 1 -type d -empty -print0 2> /dev/null)
-
-            # If no empty dirs found, we're done with this location
-            if [[ ${#nested_empty_dirs[@]} -eq 0 ]]; then
-                break
-            fi
-
-            local location_name=$(basename "$location")
-            safe_clean "${nested_empty_dirs[@]}" "Empty $location_name subdirs"
-
-            ((iteration++))
-        done
-    done
-
-    # Empty file cleanup is skipped to avoid removing app sentinel files.
-}
-
 # Remove old Google Chrome versions while keeping Current.
 clean_chrome_old_versions() {
     local -a app_paths=(
@@ -84,8 +19,8 @@ clean_chrome_old_versions() {
         "$HOME/Applications/Google Chrome.app"
     )
 
-    # Use -f to match Chrome Helper processes as well
-    if pgrep -f "Google Chrome" > /dev/null 2>&1; then
+    # Match the exact Chrome process name to avoid false positives
+    if pgrep -x "Google Chrome" > /dev/null 2>&1; then
         echo -e "  ${YELLOW}${ICON_WARNING}${NC} Google Chrome running · old versions cleanup skipped"
         return 0
     fi
@@ -164,8 +99,8 @@ clean_edge_old_versions() {
         "$HOME/Applications/Microsoft Edge.app"
     )
 
-    # Use -f to match Edge Helper processes as well
-    if pgrep -f "Microsoft Edge" > /dev/null 2>&1; then
+    # Match the exact Edge process name to avoid false positives (e.g., Microsoft Teams)
+    if pgrep -x "Microsoft Edge" > /dev/null 2>&1; then
         echo -e "  ${YELLOW}${ICON_WARNING}${NC} Microsoft Edge running · old versions cleanup skipped"
         return 0
     fi
@@ -242,7 +177,7 @@ clean_edge_updater_old_versions() {
     local updater_dir="$HOME/Library/Application Support/Microsoft/EdgeUpdater/apps/msedge-stable"
     [[ -d "$updater_dir" ]] || return 0
 
-    if pgrep -f "Microsoft Edge" > /dev/null 2>&1; then
+    if pgrep -x "Microsoft Edge" > /dev/null 2>&1; then
         echo -e "  ${YELLOW}${ICON_WARNING}${NC} Microsoft Edge running · updater cleanup skipped"
         return 0
     fi
@@ -350,17 +285,13 @@ scan_external_volumes() {
 }
 # Finder metadata (.DS_Store).
 clean_finder_metadata() {
-    stop_section_spinner
     if [[ "$PROTECT_FINDER_METADATA" == "true" ]]; then
-        note_activity
-        echo -e "  ${GREEN}${ICON_EMPTY}${NC} Finder metadata · whitelist protected"
         return
     fi
     clean_ds_store_tree "$HOME" "Home directory (.DS_Store)"
 }
 # macOS system caches and user-level leftovers.
 clean_macos_system_caches() {
-    stop_section_spinner
     # safe_clean already checks protected paths.
     safe_clean ~/Library/Saved\ Application\ State/* "Saved application states" || true
     safe_clean ~/Library/Caches/com.apple.photoanalysisd "Photo analysis cache" || true
@@ -380,7 +311,6 @@ clean_macos_system_caches() {
     safe_clean ~/Library/Application\ Support/AddressBook/Sources/*/Photos.cache "Address Book photo cache" || true
 }
 clean_recent_items() {
-    stop_section_spinner
     local shared_dir="$HOME/Library/Application Support/com.apple.sharedfilelist"
     local -a recent_lists=(
         "$shared_dir/com.apple.LSSharedFileList.RecentApplications.sfl2"
@@ -400,7 +330,6 @@ clean_recent_items() {
     safe_clean ~/Library/Preferences/com.apple.recentitems.plist "Recent items preferences" || true
 }
 clean_mail_downloads() {
-    stop_section_spinner
     local mail_age_days=${MOLE_MAIL_AGE_DAYS:-}
     if ! [[ "$mail_age_days" =~ ^[0-9]+$ ]]; then
         mail_age_days=30
@@ -511,7 +440,6 @@ process_container_cache() {
 }
 # Browser caches (Safari/Chrome/Edge/Firefox).
 clean_browsers() {
-    stop_section_spinner
     safe_clean ~/Library/Caches/com.apple.Safari/* "Safari cache"
     # Chrome/Chromium.
     safe_clean ~/Library/Caches/Google/Chrome/* "Chrome cache"
@@ -547,7 +475,6 @@ clean_browsers() {
 }
 # Cloud storage caches.
 clean_cloud_storage() {
-    stop_section_spinner
     safe_clean ~/Library/Caches/com.dropbox.* "Dropbox cache"
     safe_clean ~/Library/Caches/com.getdropbox.dropbox "Dropbox cache"
     safe_clean ~/Library/Caches/com.google.GoogleDrive "Google Drive cache"
@@ -558,7 +485,6 @@ clean_cloud_storage() {
 }
 # Office app caches.
 clean_office_applications() {
-    stop_section_spinner
     safe_clean ~/Library/Caches/com.microsoft.Word "Microsoft Word cache"
     safe_clean ~/Library/Caches/com.microsoft.Excel "Microsoft Excel cache"
     safe_clean ~/Library/Caches/com.microsoft.Powerpoint "Microsoft PowerPoint cache"
@@ -578,7 +504,6 @@ clean_virtualization_tools() {
 }
 # Application Support logs/caches.
 clean_application_support_logs() {
-    stop_section_spinner
     if [[ ! -d "$HOME/Library/Application Support" ]] || ! ls "$HOME/Library/Application Support" > /dev/null 2>&1; then
         note_activity
         echo -e "  ${YELLOW}${ICON_WARNING}${NC} Skipped: No permission to access Application Support"
