@@ -12,6 +12,11 @@ readonly MOLE_APP_PROTECTION_LOADED=1
 _MOLE_CORE_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
 [[ -z "${MOLE_BASE_LOADED:-}" ]] && source "$_MOLE_CORE_DIR/base.sh"
 
+# Declare WHITELIST_PATTERNS if not already set (used by is_path_whitelisted)
+if ! declare -p WHITELIST_PATTERNS &> /dev/null; then
+    declare -a WHITELIST_PATTERNS=()
+fi
+
 # Application Management
 
 # Critical system components protected from uninstallation
@@ -85,8 +90,9 @@ readonly DATA_PROTECTED_BUNDLES=(
     "com.lastpass.*"  # LastPass
     "com.dashlane.*"  # Dashlane
     "com.bitwarden.*" # Bitwarden
-    "com.keepassx.*"  # KeePassXC
+    "com.keepassx.*"  # KeePassXC (Legacy)
     "org.keepassx.*"  # KeePassX
+    "org.keepassxc.*" # KeePassXC
     "com.authy.*"     # Authy
     "com.yubico.*"    # YubiKey Manager
 
@@ -413,6 +419,10 @@ readonly DATA_PROTECTED_BUNDLES=(
     "tv.plex.player.desktop"   # Plex
     "com.netease.163music"     # NetEase Music
 
+    # Web Browsers (protect complex storage like IndexedDB, localStorage)
+    "Firefox"       # Firefox Application Support
+    "org.mozilla.*" # Firefox bundle IDs
+
     # License Management & App Stores
     "com.paddle.Paddle*"          # Paddle (license management)
     "com.setapp.DesktopClient"    # Setapp
@@ -633,6 +643,14 @@ is_path_whitelisted() {
 find_app_files() {
     local bundle_id="$1"
     local app_name="$2"
+
+    # Early validation: require at least one valid identifier
+    # Skip scanning if both bundle_id and app_name are invalid
+    if [[ -z "$bundle_id" || "$bundle_id" == "unknown" ]] &&
+        [[ -z "$app_name" || ${#app_name} -lt 2 ]]; then
+        return 0 # Silent return to avoid invalid scanning
+    fi
+
     local -a files_to_clean=()
 
     # Normalize app name for matching
@@ -730,10 +748,17 @@ find_app_files() {
     fi
 
     # Launch Agents by name (special handling)
-    if [[ ${#app_name} -gt 3 ]] && [[ -d ~/Library/LaunchAgents ]]; then
+    # Note: LaunchDaemons are system-level and handled in find_app_system_files()
+    # Minimum 5-char threshold prevents false positives (e.g., "Time" matching system agents)
+    # Short-name apps (e.g., Zoom, Arc) are still cleaned via bundle_id matching above
+    if [[ ${#app_name} -ge 5 ]] && [[ -d ~/Library/LaunchAgents ]]; then
         while IFS= read -r -d '' plist; do
+            local plist_name=$(basename "$plist")
+            if [[ "$plist_name" =~ ^com\.apple\. ]]; then
+                continue
+            fi
             files_to_clean+=("$plist")
-        done < <(command find ~/Library/LaunchAgents -maxdepth 1 \( -name "*$app_name*.plist" \) -print0 2> /dev/null)
+        done < <(command find ~/Library/LaunchAgents -maxdepth 1 -name "*$app_name*.plist" -print0 2> /dev/null)
     fi
 
     # Handle specialized toolchains and development environments
@@ -746,7 +771,7 @@ find_app_files() {
 
     # 2. Android Studio (Google)
     if [[ "$app_name" =~ Android.*Studio|android.*studio ]] || [[ "$bundle_id" =~ google.*android.*studio|jetbrains.*android ]]; then
-        for d in ~/AndroidStudioProjects ~/Library/Android ~/.android ~/.gradle; do
+        for d in ~/AndroidStudioProjects ~/Library/Android ~/.android; do
             [[ -d "$d" ]] && files_to_clean+=("$d")
         done
         [[ -d ~/Library/Application\ Support/Google ]] && while IFS= read -r -d '' d; do files_to_clean+=("$d"); done < <(command find ~/Library/Application\ Support/Google -maxdepth 1 -name "AndroidStudio*" -print0 2> /dev/null)
@@ -859,12 +884,24 @@ find_app_system_files() {
         done < <(command find /private/var/db/receipts -maxdepth 1 \( -name "*$bundle_id*" \) -print0 2> /dev/null)
     fi
 
+    local receipt_files=""
+    receipt_files=$(find_app_receipt_files "$bundle_id")
+
+    local combined_files=""
     if [[ ${#system_files[@]} -gt 0 ]]; then
-        printf '%s\n' "${system_files[@]}"
+        combined_files=$(printf '%s\n' "${system_files[@]}")
     fi
 
-    # Find files from receipts (Deep Scan)
-    find_app_receipt_files "$bundle_id"
+    if [[ -n "$receipt_files" ]]; then
+        if [[ -n "$combined_files" ]]; then
+            combined_files+=$'\n'
+        fi
+        combined_files+="$receipt_files"
+    fi
+
+    if [[ -n "$combined_files" ]]; then
+        printf '%s\n' "$combined_files" | sort -u
+    fi
 }
 
 # Locate files using installation receipts (BOM)
@@ -874,6 +911,13 @@ find_app_receipt_files() {
     # Skip if no bundle ID
     [[ -z "$bundle_id" || "$bundle_id" == "unknown" ]] && return 0
 
+    # Validate bundle_id format to prevent wildcard injection
+    # Only allow alphanumeric characters, dots, hyphens, and underscores
+    if [[ ! "$bundle_id" =~ ^[a-zA-Z0-9._-]+$ ]]; then
+        debug_log "Invalid bundle_id format: $bundle_id"
+        return 0
+    fi
+
     local -a receipt_files=()
     local -a bom_files=()
 
@@ -905,49 +949,51 @@ find_app_receipt_files() {
                     clean_path="/$clean_path"
                 fi
 
+                # Path traversal protection: reject paths containing ..
+                if [[ "$clean_path" =~ \.\. ]]; then
+                    debug_log "Rejected path traversal in BOM: $clean_path"
+                    continue
+                fi
+
+                # Normalize path (remove duplicate slashes)
+                clean_path=$(tr -s "/" <<< "$clean_path")
+
                 # ------------------------------------------------------------------------
                 # Safety check: restrict removal to trusted paths
                 # ------------------------------------------------------------------------
                 local is_safe=false
 
-                # Whitelisted prefixes
+                # Whitelisted prefixes (exclude /Users, /usr, /opt)
                 case "$clean_path" in
                     /Applications/*) is_safe=true ;;
-                    /Users/*) is_safe=true ;;
-                    /usr/local/*) is_safe=true ;;
-                    /opt/*) is_safe=true ;;
-                    /Library/*)
-                        # Filter sub-paths in /Library to avoid system damage
-                        # Allow safely: Application Support, Caches, Logs, Preferences
-                        case "$clean_path" in
-                            /Library/Application\ Support/*) is_safe=true ;;
-                            /Library/Caches/*) is_safe=true ;;
-                            /Library/Logs/*) is_safe=true ;;
-                            /Library/Preferences/*) is_safe=true ;;
-                            /Library/PrivilegedHelperTools/*) is_safe=true ;;
-                            /Library/LaunchAgents/*) is_safe=true ;;
-                            /Library/LaunchDaemons/*) is_safe=true ;;
-                            /Library/Internet\ Plug-Ins/*) is_safe=true ;;
-                            /Library/Audio/Plug-Ins/*) is_safe=true ;;
-                            /Library/Extensions/*) is_safe=false ;; # Default unsafe
-                            *) is_safe=false ;;
-                        esac
-                        ;;
+                    /Library/Application\ Support/*) is_safe=true ;;
+                    /Library/Caches/*) is_safe=true ;;
+                    /Library/Logs/*) is_safe=true ;;
+                    /Library/Preferences/*) is_safe=true ;;
+                    /Library/LaunchAgents/*) is_safe=true ;;
+                    /Library/LaunchDaemons/*) is_safe=true ;;
+                    /Library/PrivilegedHelperTools/*) is_safe=true ;;
+                    /Library/Extensions/*) is_safe=false ;;
+                    *) is_safe=false ;;
                 esac
 
                 # Hard blocks
                 case "$clean_path" in
-                    /System/* | /usr/bin/* | /usr/lib/* | /bin/* | /sbin/*) is_safe=false ;;
+                    /System/* | /usr/bin/* | /usr/lib/* | /bin/* | /sbin/* | /private/*) is_safe=false ;;
                 esac
 
                 if [[ "$is_safe" == "true" && -e "$clean_path" ]]; then
-                    # If lsbom lists /Applications, skip to avoid system damage.
-                    # Extra check: path must be deep enough?
-                    # If path is just "/Applications", skip.
-                    if [[ "$clean_path" == "/Applications" || "$clean_path" == "/Library" || "$clean_path" == "/usr/local" ]]; then
+                    # Skip top-level directories
+                    if [[ "$clean_path" == "/Applications" || "$clean_path" == "/Library" ]]; then
                         continue
                     fi
 
+                    if declare -f should_protect_path > /dev/null 2>&1; then
+                        if should_protect_path "$clean_path"; then
+                            continue
+                        fi
+                    fi
+
                     receipt_files+=("$clean_path")
                 fi
 
