@@ -201,7 +201,6 @@ paginated_multi_select() {
         export MOLE_MENU_SORT_MODE="$sort_mode"
         export MOLE_MENU_SORT_REVERSE="$sort_reverse"
         restore_terminal
-        unset MOLE_READ_KEY_FORCE_CHAR
     }
 
     # Interrupt handler
@@ -595,7 +594,6 @@ paginated_multi_select() {
             "QUIT")
                 if [[ "$filter_mode" == "true" ]]; then
                     filter_mode="false"
-                    unset MOLE_READ_KEY_FORCE_CHAR
                     filter_query=""
                     applied_query=""
                     top_index=0
@@ -632,10 +630,29 @@ paginated_multi_select() {
                     prev_cursor_pos=$cursor_pos
                     continue # Skip full redraw
                 elif [[ $top_index -gt 0 ]]; then
+                    # Scroll up - redraw visible items only
                     ((top_index--))
+
+                    # Redraw all visible items (faster than full screen redraw)
+                    local start_idx=$top_index
+                    local end_idx=$((top_index + items_per_page - 1))
+                    local visible_total=${#view_indices[@]}
+                    [[ $end_idx -ge $visible_total ]] && end_idx=$((visible_total - 1))
+
+                    for ((i = start_idx; i <= end_idx; i++)); do
+                        local row=$((i - start_idx + 3)) # +3 for header
+                        printf "\033[%d;1H" "$row" >&2
+                        local is_current=false
+                        [[ $((i - start_idx)) -eq $cursor_pos ]] && is_current=true
+                        render_item $((i - start_idx)) $is_current
+                    done
+
+                    # Move cursor to footer
+                    printf "\033[%d;1H" "$((items_per_page + 4))" >&2
+
                     prev_cursor_pos=$cursor_pos
                     prev_top_index=$top_index
-                    need_full_redraw=true # Scrolling requires full redraw
+                    continue
                 fi
                 ;;
             "DOWN")
@@ -670,15 +687,34 @@ paginated_multi_select() {
                             prev_cursor_pos=$cursor_pos
                             continue # Skip full redraw
                         elif [[ $((top_index + visible_count)) -lt ${#view_indices[@]} ]]; then
+                            # Scroll down - redraw visible items only
                             ((top_index++))
                             visible_count=$((${#view_indices[@]} - top_index))
                             [[ $visible_count -gt $items_per_page ]] && visible_count=$items_per_page
                             if [[ $cursor_pos -ge $visible_count ]]; then
                                 cursor_pos=$((visible_count - 1))
                             fi
+
+                            # Redraw all visible items (faster than full screen redraw)
+                            local start_idx=$top_index
+                            local end_idx=$((top_index + items_per_page - 1))
+                            local visible_total=${#view_indices[@]}
+                            [[ $end_idx -ge $visible_total ]] && end_idx=$((visible_total - 1))
+
+                            for ((i = start_idx; i <= end_idx; i++)); do
+                                local row=$((i - start_idx + 3)) # +3 for header
+                                printf "\033[%d;1H" "$row" >&2
+                                local is_current=false
+                                [[ $((i - start_idx)) -eq $cursor_pos ]] && is_current=true
+                                render_item $((i - start_idx)) $is_current
+                            done
+
+                            # Move cursor to footer
+                            printf "\033[%d;1H" "$((items_per_page + 4))" >&2
+
                             prev_cursor_pos=$cursor_pos
                             prev_top_index=$top_index
-                            need_full_redraw=true # Scrolling requires full redraw
+                            continue
                         fi
                     fi
                 fi
@@ -726,7 +762,9 @@ paginated_multi_select() {
                 if [[ "$filter_mode" == "true" ]]; then
                     local ch="${key#CHAR:}"
                     filter_query+="$ch"
+                    rebuild_view
                     need_full_redraw=true
+                    continue
                 elif [[ "$has_metadata" == "true" ]]; then
                     # Cycle sort mode (only if metadata available)
                     case "$sort_mode" in
@@ -751,7 +789,6 @@ paginated_multi_select() {
                 else
                     # Enter filter mode
                     filter_mode="true"
-                    export MOLE_READ_KEY_FORCE_CHAR=1
                     filter_query=""
                     top_index=0
                     cursor_pos=0
@@ -777,6 +814,9 @@ paginated_multi_select() {
                     fi
                 else
                     filter_query+="j"
+                    rebuild_view
+                    need_full_redraw=true
+                    continue
                 fi
                 ;;
             "CHAR:k")
@@ -791,17 +831,66 @@ paginated_multi_select() {
                     fi
                 else
                     filter_query+="k"
+                    rebuild_view
+                    need_full_redraw=true
+                    continue
+                fi
+                ;;
+            "TOUCHID")
+                if [[ "$filter_mode" == "true" ]]; then
+                    filter_query+="t"
+                    rebuild_view
+                    need_full_redraw=true
+                    continue
+                fi
+                ;;
+            "RIGHT")
+                if [[ "$filter_mode" == "true" ]]; then
+                    filter_query+="l"
+                    rebuild_view
+                    need_full_redraw=true
+                    continue
+                fi
+                ;;
+            "LEFT")
+                if [[ "$filter_mode" == "true" ]]; then
+                    filter_query+="h"
+                    rebuild_view
+                    need_full_redraw=true
+                    continue
+                fi
+                ;;
+            "MORE")
+                if [[ "$filter_mode" == "true" ]]; then
+                    filter_query+="m"
+                    rebuild_view
+                    need_full_redraw=true
+                    continue
+                fi
+                ;;
+            "UPDATE")
+                if [[ "$filter_mode" == "true" ]]; then
+                    filter_query+="u"
+                    rebuild_view
+                    need_full_redraw=true
+                    continue
                 fi
                 ;;
             "CHAR:f" | "CHAR:F")
                 if [[ "$filter_mode" == "true" ]]; then
                     filter_query+="${key#CHAR:}"
+                    rebuild_view
+                    need_full_redraw=true
+                    continue
                 fi
                 # F is currently unbound in normal mode to avoid conflict with Refresh (R)
                 ;;
             "CHAR:r" | "CHAR:R")
                 if [[ "$filter_mode" == "true" ]]; then
                     filter_query+="${key#CHAR:}"
+                    rebuild_view
+                    need_full_redraw=true
+                    continue
                 else
                     # Trigger Refresh signal (Unified with Analyze)
                     cleanup
@@ -811,6 +900,9 @@ paginated_multi_select() {
             "CHAR:o" | "CHAR:O")
                 if [[ "$filter_mode" == "true" ]]; then
                     filter_query+="${key#CHAR:}"
+                    rebuild_view
+                    need_full_redraw=true
+                    continue
                 elif [[ "$has_metadata" == "true" ]]; then
                     # O toggles reverse order (Unified Sort Order)
                     if [[ "$sort_reverse" == "true" ]]; then
@@ -826,7 +918,11 @@ paginated_multi_select() {
                 # Backspace filter
                 if [[ "$filter_mode" == "true" && -n "$filter_query" ]]; then
                     filter_query="${filter_query%?}"
+                    # Rebuild view to apply filter in real-time
+                    rebuild_view
+                    # Trigger redraw and continue to avoid drain_pending_input
                     need_full_redraw=true
+                    continue
                 fi
                 ;;
             CHAR:*)
@@ -835,7 +931,11 @@ paginated_multi_select() {
                     # avoid accidental leading spaces
                     if [[ -n "$filter_query" || "$ch" != " " ]]; then
                         filter_query+="$ch"
+                        # Rebuild view to apply filter in real-time
+                        rebuild_view
+                        # Trigger redraw and continue to avoid drain_pending_input
                         need_full_redraw=true
+                        continue
                     fi
                 fi
                 ;;
@@ -843,17 +943,9 @@ paginated_multi_select() {
                 if [[ "$filter_mode" == "true" ]]; then
                     applied_query="$filter_query"
                     filter_mode="false"
-                    unset MOLE_READ_KEY_FORCE_CHAR
-                    top_index=0
-                    cursor_pos=0
-
-                    searching="true"
-                    draw_menu           # paint "searching..."
-                    drain_pending_input # drop any extra keypresses (e.g., double-Enter)
+                    # Preserve cursor/top_index so navigation during search is respected
                     rebuild_view
-                    searching="false"
-                    draw_menu
-                    continue
+                    # Fall through to confirmation logic
                 fi
                 # In normal mode: smart Enter behavior
                 # 1. Check if any items are already selected
