@@ -3,29 +3,39 @@
 set -euo pipefail
 
 # Ensure common.sh is loaded.
-SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")/.." && pwd)"
+SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")/../.." && pwd)"
 [[ -z "${MOLE_COMMON_LOADED:-}" ]] && source "$SCRIPT_DIR/lib/core/common.sh"
 
+# Load Homebrew cask support (provides get_brew_cask_name, brew_uninstall_cask)
+[[ -f "$SCRIPT_DIR/lib/uninstall/brew.sh" ]] && source "$SCRIPT_DIR/lib/uninstall/brew.sh"
+
 # Batch uninstall with a single confirmation.
 
-# User data detection patterns (prompt user to backup if found).
-readonly SENSITIVE_DATA_PATTERNS=(
-    "\.warp"                               # Warp terminal configs/themes
-    "/\.config/"                           # Standard Unix config directory
-    "/themes/"                             # Theme customizations
-    "/settings/"                           # Settings directories
-    "/Application Support/[^/]+/User Data" # Chrome/Electron user data
-    "/Preferences/[^/]+\.plist"            # User preference files
-    "/Documents/"                          # User documents
-    "/\.ssh/"                              # SSH keys and configs (critical)
-    "/\.gnupg/"                            # GPG keys (critical)
-)
+# High-performance sensitive data detection (pure Bash, no subprocess)
+# Faster than grep for batch operations, especially when processing many apps
+has_sensitive_data() {
+    local files="$1"
+    [[ -z "$files" ]] && return 1
 
-# Join patterns into a single regex for grep.
-SENSITIVE_DATA_REGEX=$(
-    IFS='|'
-    echo "${SENSITIVE_DATA_PATTERNS[*]}"
-)
+    while IFS= read -r file; do
+        [[ -z "$file" ]] && continue
+
+        # Use Bash native pattern matching (faster than spawning grep)
+        case "$file" in
+            */.warp* | */.config/* | */themes/* | */settings/* | */User\ Data/* | \
+                */.ssh/* | */.gnupg/* | */Documents/* | */Preferences/*.plist | \
+                */Desktop/* | */Downloads/* | */Movies/* | */Music/* | */Pictures/* | \
+                */.password* | */.token* | */.auth* | */keychain* | \
+                */Passwords/* | */Accounts/* | */Cookies/* | \
+                */.aws/* | */.docker/config.json | */.kube/* | \
+                */credentials/* | */secrets/*)
+                return 0 # Found sensitive data
+                ;;
+        esac
+    done <<< "$files"
+
+    return 1 # Not found
+}
 
 # Decode and validate base64 file list (safe for set -e).
 decode_file_list() {
@@ -88,6 +98,42 @@ stop_launch_services() {
     fi
 }
 
+# Remove macOS Login Items for an app
+remove_login_item() {
+    local app_name="$1"
+    local bundle_id="$2"
+
+    # Skip if no identifiers provided
+    [[ -z "$app_name" && -z "$bundle_id" ]] && return 0
+
+    # Strip .app suffix if present (login items don't include it)
+    local clean_name="${app_name%.app}"
+
+    # Remove from Login Items using index-based deletion (handles broken items)
+    if [[ -n "$clean_name" ]]; then
+        # Escape double quotes and backslashes for AppleScript
+        local escaped_name="${clean_name//\\/\\\\}"
+        escaped_name="${escaped_name//\"/\\\"}"
+
+        osascript <<- EOF > /dev/null 2>&1 || true
+			tell application "System Events"
+			    try
+			        set itemCount to count of login items
+			        -- Delete in reverse order to avoid index shifting
+			        repeat with i from itemCount to 1 by -1
+			            try
+			                set itemName to name of login item i
+			                if itemName is "$escaped_name" then
+			                    delete login item i
+			                end if
+			            end try
+			        end repeat
+			    end try
+			end tell
+		EOF
+    fi
+}
+
 # Remove files (handles symlinks, optional sudo).
 remove_file_list() {
     local file_list="$1"
@@ -97,17 +143,21 @@ remove_file_list() {
     while IFS= read -r file; do
         [[ -n "$file" && -e "$file" ]] || continue
 
+        if ! validate_path_for_deletion "$file"; then
+            continue
+        fi
+
         if [[ -L "$file" ]]; then
             if [[ "$use_sudo" == "true" ]]; then
-                sudo rm "$file" 2> /dev/null && ((count++)) || true
+                sudo rm "$file" 2> /dev/null && ((++count)) || true
             else
-                rm "$file" 2> /dev/null && ((count++)) || true
+                rm "$file" 2> /dev/null && ((++count)) || true
             fi
         else
             if [[ "$use_sudo" == "true" ]]; then
-                safe_sudo_remove "$file" && ((count++)) || true
+                safe_sudo_remove "$file" && ((++count)) || true
             else
-                safe_remove "$file" true && ((count++)) || true
+                safe_remove "$file" true && ((++count)) || true
             fi
         fi
     done <<< "$file_list"
@@ -146,7 +196,15 @@ batch_uninstall_applications() {
             running_apps+=("$app_name")
         fi
 
-        # Sudo needed if bundle owner/dir is not writable or system files exist.
+        # Check if it's a Homebrew cask (deterministic: resolved path in Caskroom)
+        local cask_name=""
+        cask_name=$(get_brew_cask_name "$app_path" || echo "")
+        local is_brew_cask="false"
+        [[ -n "$cask_name" ]] && is_brew_cask="true"
+
+        # Full file scanning for ALL apps (including Homebrew casks)
+        # brew uninstall --cask does NOT remove user data (caches, prefs, app support)
+        # Mole's value is cleaning those up, so we must scan for them
         local needs_sudo=false
         local app_owner=$(get_file_owner "$app_path")
         local current_user=$(whoami)
@@ -179,7 +237,7 @@ batch_uninstall_applications() {
 
         # Check for sensitive user data once.
         local has_sensitive_data="false"
-        if [[ -n "$related_files" ]] && echo "$related_files" | grep -qE "$SENSITIVE_DATA_REGEX"; then
+        if has_sensitive_data "$related_files"; then
             has_sensitive_data="true"
         fi
 
@@ -188,7 +246,7 @@ batch_uninstall_applications() {
         encoded_files=$(printf '%s' "$related_files" | base64 | tr -d '\n')
         local encoded_system_files
         encoded_system_files=$(printf '%s' "$system_files" | base64 | tr -d '\n')
-        app_details+=("$app_name|$app_path|$bundle_id|$total_kb|$encoded_files|$encoded_system_files|$has_sensitive_data|$needs_sudo")
+        app_details+=("$app_name|$app_path|$bundle_id|$total_kb|$encoded_files|$encoded_system_files|$has_sensitive_data|$needs_sudo|$is_brew_cask|$cask_name")
     done
     if [[ -t 1 ]]; then stop_inline_spinner; fi
 
@@ -214,12 +272,17 @@ batch_uninstall_applications() {
     fi
 
     for detail in "${app_details[@]}"; do
-        IFS='|' read -r app_name app_path bundle_id total_kb encoded_files encoded_system_files has_sensitive_data needs_sudo_flag <<< "$detail"
-        local related_files=$(decode_file_list "$encoded_files" "$app_name")
-        local system_files=$(decode_file_list "$encoded_system_files" "$app_name")
+        IFS='|' read -r app_name app_path bundle_id total_kb encoded_files encoded_system_files has_sensitive_data needs_sudo_flag is_brew_cask cask_name <<< "$detail"
         local app_size_display=$(bytes_to_human "$((total_kb * 1024))")
 
-        echo -e "${BLUE}${ICON_CONFIRM}${NC} ${app_name} ${GRAY}(${app_size_display})${NC}"
+        local brew_tag=""
+        [[ "$is_brew_cask" == "true" ]] && brew_tag=" ${CYAN}[Brew]${NC}"
+        echo -e "${BLUE}${ICON_CONFIRM}${NC} ${app_name}${brew_tag} ${GRAY}(${app_size_display})${NC}"
+
+        # Show detailed file list for ALL apps (brew casks leave user data behind)
+        local related_files=$(decode_file_list "$encoded_files" "$app_name")
+        local system_files=$(decode_file_list "$encoded_system_files" "$app_name")
+
         echo -e "  ${GREEN}${ICON_SUCCESS}${NC} ${app_path/$HOME/~}"
 
         # Show related files (limit to 5).
@@ -275,7 +338,7 @@ batch_uninstall_applications() {
             return 0
             ;;
         "" | $'\n' | $'\r' | y | Y)
-            printf "\r\033[K" # Clear the prompt line
+            echo "" # Move to next line
             ;;
         *)
             echo ""
@@ -305,31 +368,61 @@ batch_uninstall_applications() {
         sudo_keepalive_pid=$!
     fi
 
-    if [[ -t 1 ]]; then start_inline_spinner "Uninstalling apps..."; fi
-
-    # Perform uninstallations (silent mode, show results at end).
-    if [[ -t 1 ]]; then stop_inline_spinner; fi
+    # Perform uninstallations with per-app progress feedback
     local success_count=0 failed_count=0
+    local brew_apps_removed=0 # Track successful brew uninstalls for autoremove tip
     local -a failed_items=()
     local -a success_items=()
+    local current_index=0
     for detail in "${app_details[@]}"; do
-        IFS='|' read -r app_name app_path bundle_id total_kb encoded_files encoded_system_files has_sensitive_data needs_sudo <<< "$detail"
+        ((current_index++))
+        IFS='|' read -r app_name app_path bundle_id total_kb encoded_files encoded_system_files has_sensitive_data needs_sudo is_brew_cask cask_name <<< "$detail"
         local related_files=$(decode_file_list "$encoded_files" "$app_name")
         local system_files=$(decode_file_list "$encoded_system_files" "$app_name")
         local reason=""
 
+        # Show progress for current app
+        local brew_tag=""
+        [[ "$is_brew_cask" == "true" ]] && brew_tag=" ${CYAN}[Brew]${NC}"
+        if [[ -t 1 ]]; then
+            if [[ ${#app_details[@]} -gt 1 ]]; then
+                start_inline_spinner "[$current_index/${#app_details[@]}] Uninstalling ${app_name}${brew_tag}..."
+            else
+                start_inline_spinner "Uninstalling ${app_name}${brew_tag}..."
+            fi
+        fi
+
         # Stop Launch Agents/Daemons before removal.
         local has_system_files="false"
         [[ -n "$system_files" ]] && has_system_files="true"
+
         stop_launch_services "$bundle_id" "$has_system_files"
 
+        # Remove from Login Items
+        remove_login_item "$app_name" "$bundle_id"
+
         if ! force_kill_app "$app_name" "$app_path"; then
             reason="still running"
         fi
 
         # Remove the application only if not running.
+        local used_brew_successfully=false
         if [[ -z "$reason" ]]; then
-            if [[ "$needs_sudo" == true ]]; then
+            if [[ "$is_brew_cask" == "true" && -n "$cask_name" ]]; then
+                # Stop spinner before brew output
+                [[ -t 1 ]] && stop_inline_spinner
+                # Use brew_uninstall_cask helper (handles env vars, timeout, verification)
+                if brew_uninstall_cask "$cask_name" "$app_path"; then
+                    used_brew_successfully=true
+                else
+                    # Fallback to manual removal if brew fails
+                    if [[ "$needs_sudo" == true ]]; then
+                        safe_sudo_remove "$app_path" || reason="remove failed"
+                    else
+                        safe_remove "$app_path" true || reason="remove failed"
+                    fi
+                fi
+            elif [[ "$needs_sudo" == true ]]; then
                 if ! safe_sudo_remove "$app_path"; then
                     local app_owner=$(get_file_owner "$app_path")
                     local current_user=$(whoami)
@@ -347,7 +440,13 @@ batch_uninstall_applications() {
         # Remove related files if app removal succeeded.
         if [[ -z "$reason" ]]; then
             remove_file_list "$related_files" "false" > /dev/null
-            remove_file_list "$system_files" "true" > /dev/null
+
+            # If brew successfully uninstalled the cask, avoid deleting
+            # system-level files Mole discovered. Brew manages its own
+            # receipts/symlinks and we don't want to fight it.
+            if [[ "$used_brew_successfully" != "true" ]]; then
+                remove_file_list "$system_files" "true" > /dev/null
+            fi
 
             # Clean up macOS defaults (preference domains).
             if [[ -n "$bundle_id" && "$bundle_id" != "unknown" ]]; then
@@ -361,12 +460,33 @@ batch_uninstall_applications() {
                 fi
             fi
 
+            # Stop spinner and show success
+            if [[ -t 1 ]]; then
+                stop_inline_spinner
+                if [[ ${#app_details[@]} -gt 1 ]]; then
+                    echo -e "\r\033[K${GREEN}✓${NC} [$current_index/${#app_details[@]}] ${app_name}"
+                else
+                    echo -e "\r\033[K${GREEN}✓${NC} ${app_name}"
+                fi
+            fi
+
             ((total_size_freed += total_kb))
             ((success_count++))
+            [[ "$used_brew_successfully" == "true" ]] && ((brew_apps_removed++))
             ((files_cleaned++))
             ((total_items++))
             success_items+=("$app_name")
         else
+            # Stop spinner and show failure
+            if [[ -t 1 ]]; then
+                stop_inline_spinner
+                if [[ ${#app_details[@]} -gt 1 ]]; then
+                    echo -e "\r\033[K${RED}✗${NC} [$current_index/${#app_details[@]}] ${app_name} ${GRAY}($reason)${NC}"
+                else
+                    echo -e "\r\033[K${RED}✗${NC} ${app_name} failed: $reason"
+                fi
+            fi
+
             ((failed_count++))
             failed_items+=("$app_name:$reason")
         fi
@@ -454,9 +574,32 @@ batch_uninstall_applications() {
         title="Uninstall incomplete"
     fi
 
+    echo ""
     print_summary_block "$title" "${summary_details[@]}"
     printf '\n'
 
+    # Auto-run brew autoremove if Homebrew casks were uninstalled
+    if [[ $brew_apps_removed -gt 0 ]]; then
+        # Show spinner while checking for orphaned dependencies
+        if [[ -t 1 ]]; then
+            start_inline_spinner "Checking brew dependencies..."
+        fi
+
+        local autoremove_output removed_count
+        autoremove_output=$(HOMEBREW_NO_ENV_HINTS=1 brew autoremove 2> /dev/null) || true
+        removed_count=$(printf '%s\n' "$autoremove_output" | grep -c "^Uninstalling" || true)
+        removed_count=${removed_count:-0}
+
+        if [[ -t 1 ]]; then
+            stop_inline_spinner
+        fi
+
+        if [[ $removed_count -gt 0 ]]; then
+            echo -e "${GREEN}${ICON_SUCCESS}${NC} Cleaned $removed_count orphaned brew dependencies"
+            echo ""
+        fi
+    fi
+
     # Clean up Dock entries for uninstalled apps.
     if [[ $success_count -gt 0 ]]; then
         local -a removed_paths=()
